import matplotlib.pyplot as plt
from .file_handler import OpenFiles
import numpy as np
from skimage.feature import graycomatrix, graycoprops

class CONSTRASTCalculator:
    """Calculates image contrast using GLCM (Gray-Level Co-occurrence Matrix)."""

    def __init__(self, files: OpenFiles):
        if files.nir_band is None:
            raise ValueError("NIR band is required for contrast calculation.")
        self.files = files
        self.nir = files.nir_band

    def calculate(self, extra_params=None):
        """Calculate contrast using GLCM."""
        gray_image = np.uint8(self.nir / np.max(self.nir) * 255)  # Normalize to 8-bit
        glcm = graycomatrix(gray_image, distances=[1], angles=[0], levels=256, symmetric=True, normed=True)
        contrast = graycoprops(glcm, 'contrast')[0, 0]
        return contrast

    def export(self, file_path: str, title: str):
        plt.figure(figsize=(10, 10))
        plt.title(title)
        plt.imshow(self.nir, cmap="gray")
        plt.axis("off")
        plt.savefig(file_path)
        plt.close()



import numpy as np
import matplotlib.pyplot as plt


class Landsat:

    def __init__(self, *bands):
        for band in bands:
            if not isinstance(band, np.ndarray):
                raise ValueError("Invalid band data. Must be a NumPy array.")
        self.bands = bands
        self.stack = None

    def get_multi_spectral_img(self):
        band_list = []
        for band in self.bands:
            normalize_band = (band - np.min(band)) / (np.max(band) - np.min(band))
            band_list.append(normalize_band)
        stack = np.stack(band_list, axis=2)
        self.stack = stack
        return stack

    def show_img(self, title):
        if self.stack is None:
            self.get_multi_spectral_img()
        plt.imshow(self.stack)
        plt.title(title)
        plt.axis("off")
        plt.show()



from pathlib import Path

import cv2
from matplotlib import pyplot as plt

from app.openrs.base import Base
from app.openrs.file_handler import OpenFiles


class MEANCalculator(Base):
    """
    MEAN calculator class
    """

    def __init__(self, files: OpenFiles):
        # Check requirment bands
        if files.tif_file is None:
            raise Exception("Message")
        super().__init__(files)
        # Normalize the bands
        self.normalized_bands = files.get_normalize_bands()

        self.image = files.tif_file_metadata["image"]

    def calculate(self, extra_params: dict):
        self.mean_image = cv2.blur(self.image, (9, 9))

        return self.mean_image

    def export(self, file_path: Path, title: str):
        plt.figure(figsize=(10, 10))
        plt.imshow(self.mean_image, cmap="gray")
        plt.title(title)
        plt.axis("off")
        plt.savefig(file_path)
        plt.close()


# Export (on test mode)
if __name__ == "__main__":
    tif_file = Path.cwd() / "app/openrs/data/pan_img.tif"
    calculator = MEANCalculator(OpenFiles(tif_file=tif_file))
    mean_filter = calculator.calculate(None)
    plt.figure(figsize=(10, 10))
    plt.imshow(mean_filter, cmap="gray")
    plt.title("Mean Filter")
    plt.axis("off")
    plt.show()



from .kmeans import KMEANSCalculator
from .contrast import CONSTRASTCalculator
from .gaussian import GAUSSIANCalculator
from .laplacian import LAPLACIANCalculator
from .mean import MEANCalculator
from .median import MEDIANCalculator
from .sobel import SOBELCalculator
from .file_handler import OpenFiles
from .OException import OException
